

####################### Analyse de la variance (ANOVA) ###############################################
# Une variable quantitative distribue dans des groupes ("traitement"),
# donc dans des modalits (ou niveau) d'une (ou plusieurs) variables qualitatives (ou facteur).
# L'ANOVA dtermine s'il y a un "effet" du groupe sur l'cart  la moyenne gnrale.
# Elle permet d'analyser K>2 groupes (K=2 groupes: test de Student) 
######################################################################################################


######################################################################################################
####################### ANOVA   un facteur (1 variable qualitative) #################################
######################################################################################################

###### EXEMPLE 1: Comparaison de la production journalire de lait pour 3 races de vaches laitires #########################################

# variable quantitative
A = c(20.1,19.8,21.3,20.7)
B = c(22.6,24.1,23.8,22.5,23.4,24.5,22.9)
J = c(31.2,31.6,31.0,32.1,31.4)
mean(A)
mean(B)
mean(J)
prod <- c(A,B,J)

# variable qualitative
group = as.factor(c(rep("Abondance", 4), rep("Bearnaise", 7), rep("Jersiaise", 5)))

# reprsentation graphique
boxplot(prod~group,col="red")		# ou boxplot(A,B,J,col="red",names=c("Abondance","Bearnaise","Jersiaise"))


## Tests prmiminaires (prsupposs pour l'ANOVA)

# Test de normalit
shapiro.test(A)
shapiro.test(B)
shapiro.test(J)


# Test d'galit des variances entre groupes - homoscdasticit - (H0 : sigma^2_A = sigma^2_B = sigma^2_J)
# Utilis pour tester si k chantillons sont tirs de populations dont les variances sont gales
bartlett.test(prod~group)

## ANOVA (analyse des composantes de la variances pour tester des diffrences de moyenne)
# H0: (A)= (B)= (C)
# H1: au moins une moyenne diffre des autres
# en pratique on estime var(intergroupe) et var(intragroupe) ; puis test de Fisher sur les variances (H0:var(intergroupe) = var(intragroupe))  

res = aov(prod ~ group)	### composantes de la variance
summary(res)		### tableau d'ANOVA (ou "anova(res)" ou "summary.aov(res)")
model.tables(res)		### tailles d'effet (les diffrences entre chaque groupe et la moyenne globale)
#plot(res)			### graphiques diagnostiques (homognit des variances intra-groupes, distribution normale des rsidus...)

# Vrification (a posteriori) des hypothses de l'ANOVA
layout(matrix(1:3, 1, 3))  # graphiques 1  3, sur 1 ligne et 3 colonnes
#Normalit des rsidus
#plot des rsidus
plot(residuals(res), ylab="Residus", col="blue") #plot des rsidus
abline(0, 0, col="red")
#histogramme des rsidus
hist(residuals(res), nclass=7, xlab="Residus", ylab="Frequence", xlim=c(-3,3), pro=T, col="blue", main="Analyse des residus") 
lines(seq(-3,3,le=100), dnorm(seq(-3,3,le=100), 0,  sd(residuals(res))), col="red")
#QQ-plot des rsidus empirique/thorique
qqnorm(residuals(res))            # quantiles des rsidus observs vs quantiles des rsidus thorique pour une N(0,1)
qqline(residuals(res), col="red") # une ligne passant par Q1 et Q3

## Comparaison entre groupes
t.test(A,B,var.equal = TRUE)
t.test(B,J,var.equal = TRUE)
t.test(A,J,var.equal = TRUE)  	# attention: COMPARAISON MULTIPLES !!!!
# pairwise.t.test permet d'effectuer les tests entre groupes 2  2
# et utilise une mthode d'ajustement des p-valeurs pour tests multiples
pairwise.t.test(prod,group)		# test de comparaison multiples
# mme chose qu'avec pairwise.t.test avec le test de Mann et whitney (Wilcoxon) 
# pour tester l'galit de moyenne entre 2 chantillons 
pairwise.wilcox.test(prod,group)	# non paramtrique (moins puissant ici), normalit donc student est prferrable
# Le test suivant donne un intervalle de confiance de la diffrence de moyenne sous H0, entre les groupes compars
TukeyHSD(aov(prod~group),"group")	# ou TukeyHSD(aov(res),"group"). Si intervalle contient 0, on accepte H0 au seuil 5%
# Le test suivant permet d'identifier quel(s) groupe(s) ont une moyenne qui diffre
library(laercio)
LDuncan(res,conf.level = 0.99)	# Test de Duncan: tablissement de groupes de moyennes similaires/diffrentes
#############################################################################################################################################


###### EXEMPLE 2: Comparaison de scores de rsistance recueillis pour quatre lignes indpendantes de Medicago truncatula (legumineuses plantes) ###############################

a<-read.table('resistance_plantes.txt',col.names=c('lignee','resistance')); a
attach(a);names(a) # on peut accder aux objets (ex: une variable) en tapant son nom 
table(lignee) # affiche les effectifs de chacun des groupes dfinis par lignee
hist(resistance)  # affiche un histogramme des scores, tout groupes confondus
g1<-resistance[lignee=="HM008"] # copie les scores des groupes 1, 2 et 3 dans les variables 'g1', 'g2' et 'g3'
g2<-resistance[lignee=="A17"] 
g3<-resistance[lignee=="DZA45"]
g4<-resistance[lignee=="HM013"]
shapiro.test(g1) # vrification de la normalit des chantillons
shapiro.test(g2)
shapiro.test(g3)
shapiro.test(g4)
bartlett.test(resistance~lignee) # homognit des variances (homosdasticit)
stripchart(resistance~lignee,method='stack')
plot(resistance~lignee,col="green") # affichage de "boites  moustache" de Score par Groupe 
boxplot(resistance~lignee) # affichage de "boites  moustache" de Score par Groupe
tapply(resistance,lignee,mean) # affiche les scores moyens par Groupe
barplot(tapply(resistance,lignee,mean))
tapply(resistance,lignee,summary)
res<-aov(resistance~lignee) # calcul de l'anova de resistance en fonction de lignee
summary(res) # rsultat de l'anova
model.tables(res) # tailles des effets
plot(res) # graphiques diagnostiques


t.test(g1,g2,var.equal=T) # comparaison g1 vs. g2, avec homoscdasticit (vrifie)
t.test(g1,g3,var.equal=T)
t.test(g1,g4,var.equal=T)
t.test(g2,g3,var.equal=T)
t.test(g2,g4,var.equal=T)
t.test(g3,g4,var.equal=T)
pairwise.t.test(resistance,lignee) # comparaisons multiples 2  2

LDuncan(res,conf.level = 0.99) # identification des groupes similaires
t.test(g3,c(g1,g2,g4)) # comparaison g3 vs. g1_g2_g4  ("DZA45" vs "les autres lignes")

summary(aov(resistance~lignee,subset=lignee!="DZA45")) # anova excluant la ligne DZA45

# test de Wilcoxon/Mann-Whitney (non-parametrique)  #### pas ncessaires ici car normalit des chantillons
wilcox.test(g1,g2)
wilcox.test(g1,g3)
wilcox.test(g1,g4)
wilcox.test(g2,g3)
wilcox.test(g2,g4)
wilcox.test(g3,g4) 
pairwise.wilcox.test(resistance,lignee) 
pairwise.wilcox.test(jitter(resistance),lignee) # rajouter un bruit aux valeurs pour rsoudre le problme des rangs identiques dans le test


# un graphique montrant les intervalles de confiances  95% des moyennes de chaque groupe:
IC=function(m,v) {
	# summary(as.factor(lignee)) : on a 10 mesures par ligne donc 9 ddl
	d <- qt(0.95, df=9) * sqrt(v/9) # intervalles de confiance  95%
	mp <- barplot(m, ylim=c(45,65), xpd=F)
	arrows(mp, m-d, mp, m+d, angle=90, code=3)
}
m <- tapply(resistance, lignee, mean)
v <- tapply(resistance, lignee, var)
IC(m,v)
#############################################################################################################################################




######################################################################################################
####################### ANOVA   deux facteurs (2 var qualitatives) ##################################
##  ANOVA  deux facteurs (chacun avec 2 modalits) et analyse de l'interaction entre facteurs ####### 
## plan structur par deux facteurs de groupes (2 variables entre sujets croises entre elles) #######
######################################################################################################

###### EXEMPLE 3: Effet du type d'engrais ("A"/"B") et de la temprature ("High","Medium") sur la croissance des plantes ####################

d<-read.table('croissance_plantes.txt',header=T)
attach(d);names(d)

par(mfrow=c(2,2))
boxplot(hauteur~engrais, main="engrais",ylab="hauteur (cm)")
boxplot(hauteur~temperature, main="temperature",ylab="hauteur (cm)")
boxplot(hauteur~temperature+engrais, main="engrais-temperature",ylab="hauteur (cm)",las=3)

# on teste la normalit des distributions (avant d'effectuer le test de Bartlett)
shapiro.test(hauteur[engrais=="A" & temperature=="Medium"])
shapiro.test(hauteur[engrais=="A" & temperature=="High"])
shapiro.test(hauteur[engrais=="B" & temperature=="Medium"])
shapiro.test(hauteur[engrais=="B" & temperature=="High"])

# on test l'galit des variances
bartlett.test(hauteur~temperature*engrais)

model1<-aov(hauteur~temperature*engrais)	# modle d'ANOVA o l'on teste l'effet de 2 facteurs ainsi que leur interaction)
summary(model1)
interaction.plot(temperature,engrais,hauteur, main="interaction plot")
#plot(model1)

model2<-aov(hauteur~temperature+engrais)	# modle d'ANOVA o l'on teste l'effet de 2 facteurs sans leur interaction)
summary(model2)
#plot(model2)

anova(model1,model2)				# comparaison des 2 modles avec une ANOVA sur les rsidus des 2 modles
#si pas de diffrences on garde le modle le plus simple,
#si diffrence (modle 2 significativement diffrent de modle 1) on garde le modle prsentant le plus de termes significatifs

# comparaison des engrais
t.test(hauteur[engrais=="A" & temperature=="High"],hauteur[engrais=="B" & temperature=="High"],var.equal=T)     # comparaison A/B pour temprature "High"
t.test(hauteur[engrais=="A" & temperature=="Medium"],hauteur[engrais=="B" & temperature=="Medium"],var.equal=T) # comparaison A/B pour temprature "Medium"

# comparaison des tempratures
t.test(hauteur[engrais=="A" & temperature=="Medium"],hauteur[engrais=="A" & temperature=="High"],var.equal=T)   # comparaison High/Medium pour l'engrais A
t.test(hauteur[engrais=="B" & temperature=="Medium"],hauteur[engrais=="B" & temperature=="High"],var.equal=T)   # comparaison High/Medium pour l'engrais B
# une temprature modre augmente significativement la croissance, SURTOUT en prsence de l'engrais A 

Gr=factor(c(rep("M/A",4),rep("M/B",4),rep("H/A",4),rep("H/B",4)))
pairwise.t.test(hauteur,Gr) # comparaisons multiples 2  2: attention ici certaines groupes ne sont pas comparables (M/A vs H/B, M/B vs H/A) !!
#############################################################################################################################################




###### EXEMPLE 4:  Effet du type d'organisme transform utilis ("Ecoli","Levure") et du mode d'alimentation du fermenteur ##################
###### ("continu"/"step") sur la production d'un antibiotique  ##############################################################################

d<-read.table('antibio.txt')
production<-d$V2 # rcupre la deuxime colonne
#~ a<-gl(2,10,40)		# cration du facteur a (ex: Ecoli/Levure)
organisme=as.factor(rep(c(rep('Ecoli',10), rep('Yeast',10)), 2))
#~ b<-gl(2,20,40)		# cration du facteur b (ex: continu/step)
alimentation=as.factor(c( rep('continu',20), rep('step',20) ))
table(organisme,alimentation)
tapply(production,list(a=organisme,b=alimentation),mean)

par(mfrow=c(1,3))
boxplot(production~organisme, col="green",names=c("Ecoli","Levure"))               # production en fonction de l'espce (Ecoli/Yesat)
boxplot(production~alimentation, col="green",names=c("continu","step"))            # production en fonction du fermenteur (continu/step)
boxplot(production~organisme+alimentation, col="green",names=c("Ecol/continu","Levure/continu","Ecol/step","Levure/step"))

shapiro.test(production[organisme=='Ecoli' & alimentation=='continu'])
shapiro.test(production[organisme=='Ecoli' & alimentation=='step'])
shapiro.test(production[organisme=='Yeast' & alimentation=='continu'])
shapiro.test(production[organisme=='Yeast' & alimentation=='step'])

bartlett.test(production~organisme+alimentation)

model1<-aov(production~organisme*alimentation)	# ANOVA (les deux facteurs a et b, ainsi que leur interaction sont inclus dans le modle)
summary(model1)
interaction.plot(organisme,alimentation,production)
#plot(model1)

model2<-aov(production~organisme+alimentation)	# ANOVA (les deux facteurs a et b sont inclus dans le modle)
summary(model2)
#plot(model2)
anova(model1,model2)	# si pas de diffrences on garde le modle le plus simple,
#si diffrence (modle 2 significativement diffrent de modle 1) on garde le modle prsentant le plus de termes significatifs

# Comparaisons de moyennes, en supposant variances diffrentes (test de Welch) 
t.test(production[organisme=='Ecoli' & alimentation=='continu'],production[organisme=='Ecoli' & alimentation=='step'],var.equal=T) #   comparaison fermenteurs ("continu" et "step") pour Ecoli
t.test(production[organisme=='Yeast' & alimentation=='continu'],production[organisme=='Yeast' & alimentation=='step'],var.equal=T) #   comparaison fermenteurs ("continu" et "step") pour Yeast
# une alimentation "par tape" ("step" ) du fermenteur amliore la production UNIQUEMENT pour Ecoli 
#############################################################################################################################################















































